from datetime import datetime
from typing import Optional

from settings_models._combat import SettingsModel, Field
from settings_models.validators import timezone_validator, deprecated_validator


class EnforcementSettings(SettingsModel):
    """
    Settings for enforcement service
    """
    payment_deadline_hours: int = Field(..., gt=0, le=960,
                                        description="How long someone can pay after exiting parking lot "
                                                    "(not including generally applied additional day)")
    strictness: int = Field(default=2, description="Strictness level for enforcement. Values to be defined.")
    enabled: bool = Field(..., description="If enforcement is enabled")
    ai_enabled: bool = Field(default=True, description="If the Arivo AI is enabled")
    last_edited: datetime = Field(..., description="Last time (UTC) when enabled setting was changed")
    arivo_collection_since: Optional[datetime] = \
        Field(default=None, description="Timestamp (UTC) when the Arivo collections app was installed the first time.")

    last_edited_validator = timezone_validator("last_edited")
    arivo_collection_since_validator = timezone_validator("arivo_collection_since")
    strictness_validator = deprecated_validator("strictness")


EnforcementSettingsType = Optional[EnforcementSettings]
EnforcementSettingsType.__doc__ = EnforcementSettings.__doc__



from datetime import datetime, timezone
from enum import Enum
from typing import Union, Any, Dict

from pydantic import root_validator

from settings_models._combat import SettingsModel, Field
from settings_models.validators import timezone_validator


def utcnow():
    return datetime.now(timezone.utc).replace(tzinfo=None)


class AssetVisualType(str, Enum):
    logo = "logo"
    marketing_image = "marketing_image"  # legacy
    tariff_table = "tariff_table"
    payment_methods = "payment_methods"


class AssetVisualDefinition(SettingsModel):
    """
    Settings for available assets
    """
    type: Union[AssetVisualType, str] = Field(..., description="Type of asset defining its behavior")
    value: Any = Field(description="URL or config of the asset")
    created: datetime = Field(default_factory=utcnow, description="Creation time (UTC) of the asset")

    created_validator = timezone_validator("created")

    @root_validator(pre=False)
    def ensure_created_naive(cls, values):
        if (values["type"] in [AssetVisualType.logo, AssetVisualType.marketing_image]
                and not isinstance(values["value"], str)):
            raise ValueError("Field value must be a string (URL) for logo and marketing_image types")
        return values


AssetVisualDefinitions = Dict[str, AssetVisualDefinition]
